import { AgentContext, FileSystem } from '@credo-ts/core';
import { Session, Store } from '@openwallet-foundation/askar-shared';
import { AskarStoreExportOptions, AskarStoreImportOptions, AskarStoreRotateKeyOptions } from './AskarApiOptions';
import { AskarModuleConfig } from './AskarModuleConfig';
export declare class AskarStoreManager {
    private fileSystem;
    private config;
    constructor(fileSystem: FileSystem, config: AskarModuleConfig);
    isStoreOpen(agentContext: AgentContext): boolean;
    private getStoreConfig;
    /**
     * When we create storage for a context we need to store the version record
     */
    private setCurrentFrameworkStorageVersionOnSession;
    /**
     * Deletes all storage related to a context. If on store level, meaning root agent
     * or when using database per wallet storage, the whole store will be deleted.
     * Otherwise only a profile within the store will be removed.
     */
    deleteContext(agentContext: AgentContext): Promise<void>;
    /**
     * Closes an active context. If on store level, meaning root agent
     * or when using database per wallet storage, the whole store will be closed.
     * Otherwise nothing will be done as profiles are opened on a store from higher level.
     */
    closeContext(agentContext: AgentContext): Promise<void>;
    /**
     * @throws {AskarStoreDuplicateError} if the wallet already exists
     * @throws {AskarStoreError} if another error occurs
     */
    provisionStore(agentContext: AgentContext): Promise<Store>;
    /**
     * @throws {AskarStoreNotFoundError} if the wallet does not exist
     * @throws {AskarStoreError} if another error occurs
     */
    openStore(agentContext: AgentContext): Promise<Store>;
    /**
     * Rotate the key of the current askar store.
     *
     * NOTE: multiple agent contexts (tenants) can use the same store. This method rotates the key for the whole store,
     * it is advised to only run this method on the root tenant agent when using profile per wallet database strategy.
     * After running this method you should change the store configuration in the Askar module.
     *
     * @throws {AskarStoreNotFoundError} if the wallet does not exist
     * @throws {AskarStoreError} if another error occurs
     */
    rotateStoreKey(agentContext: AgentContext, options: AskarStoreRotateKeyOptions): Promise<void>;
    /**
     * Exports the current askar store.
     *
     * NOTE: a store can contain profiles for multiple tenants. When you export a store
     * all profiles will be exported with it.
     */
    exportStore(agentContext: AgentContext, options: AskarStoreExportOptions): Promise<void>;
    /**
     * Imports from an external store config into the current askar store config.
     */
    importStore(agentContext: AgentContext, options: AskarStoreImportOptions): Promise<void>;
    /**
     * Delete the current askar store.
     *
     * NOTE: multiple agent contexts (tenants) can use the same store. This method deletes the whole store,
     * and if you're using multi-tenancy with profile per wallet it is advised to only run this method on the root tenant agent.
     *
     * @throws {AskarStoreNotFoundError} if the wallet does not exist
     * @throws {AskarStoreError} if another error occurs
     */
    deleteStore(agentContext: AgentContext): Promise<void>;
    /**
     * Close the current askar store
     */
    closeStore(agentContext: AgentContext): Promise<void>;
    private getAskarStoreConfig;
    /**
     * Run callback with a transaction. If the callback resolves the transaction
     * will be committed if the transaction is not closed yet. If the callback rejects
     * the transaction will be rolled back if the transaction is not closed yet.
     *
     * TODO: update to new `using` syntax so we don't have to use a callback
     */
    withTransaction<Return>(agentContext: AgentContext, callback: (session: Session) => Return): Promise<Awaited<Return>>;
    /**
     * Run callback with the session provided, the session will
     * be closed once the callback resolves or rejects if it is not closed yet.
     *
     * TODO: update to new `using` syntax so we don't have to use a callback
     */
    withSession<Return>(agentContext: AgentContext, callback: (session: Session) => Return): Promise<Awaited<Return>>;
    private getStore;
    private _withSession;
    getInitializedStoreWithProfile(agentContext: AgentContext): Promise<{
        profile: string | undefined;
        store: Store;
    }>;
    /**
     * Ensures a command is ran on a store level, preventing a tenant instance from
     * removing a whole store (and potentially other tennats).
     */
    private ensureStoreLevel;
    /**
     * Checks whether the current agent context is on store level
     */
    private isStoreLevel;
}
